"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackagesInTopologicalOrder = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const glob_1 = require("glob");
const toposort_1 = __importDefault(require("toposort"));
async function getPackagesInTopologicalOrder(cwd) {
    var _a, _b, _c, _d;
    const patterns = JSON.parse(await fs_1.promises.readFile(path_1.default.join(cwd, `package.json`), 'utf8'))
        .workspaces || [];
    const packageJsonPaths = await (0, glob_1.glob)(patterns.map((pattern) => path_1.default.join(pattern, 'package.json')));
    const internalPackages = await Promise.all(packageJsonPaths.map(async (packageJsonPath) => {
        const packageJson = JSON.parse(await fs_1.promises.readFile(packageJsonPath, 'utf8'));
        return {
            name: packageJson.name,
            location: packageJsonPath,
            packageJson,
        };
    }));
    const edges = [];
    const packageNames = new Set(internalPackages.map((p) => p.name));
    for (const internalPackage of internalPackages) {
        const anyInternalDependency = Object.keys({
            ...(((_a = internalPackage.packageJson) === null || _a === void 0 ? void 0 : _a.dependencies) || {}),
            ...(((_b = internalPackage.packageJson) === null || _b === void 0 ? void 0 : _b.devDependencies) || {}),
            ...(((_c = internalPackage.packageJson) === null || _c === void 0 ? void 0 : _c.optionalDependencies) || {}),
            ...(((_d = internalPackage.packageJson) === null || _d === void 0 ? void 0 : _d.peerDependencies) || {}),
        }).filter((dep) => packageNames.has(dep));
        edges.push(['.root', internalPackage.name]);
        if (anyInternalDependency.length) {
            for (const dep of anyInternalDependency) {
                edges.push([internalPackage.name, dep]);
            }
        }
    }
    const sorted = (0, toposort_1.default)(edges).slice(1).reverse();
    const result = sorted
        .map((packageName) => internalPackages.find((p) => p.name === packageName))
        .map((packageInfo) => {
        if (!packageInfo)
            throw new Error('unreachable');
        return {
            name: packageInfo.name,
            version: packageInfo.packageJson.version,
            private: !!packageInfo.packageJson.private,
            location: path_1.default.resolve(path_1.default.dirname(packageInfo.location)),
        };
    });
    return result;
}
exports.getPackagesInTopologicalOrder = getPackagesInTopologicalOrder;
//# sourceMappingURL=get-packages-in-topological-order.js.map